#include <iostream>
#include <vector>
#include <queue>
#include <iterator>
#include <fstream>

using namespace std;

#include <unistd.h>

#include "geneorder.h"
#include "modelidentifier.h"
#include "genesorter.h"
#include "sortaction.h"
#include "genealgorithms.h"
#include "model.h"

int main(int argc, char** argv){

	string ann = "default.ann";
	Model model(0);
	bool detectModel = true;

	//Parse command line arguments
	int opt;
	while ((opt = getopt(argc, argv, "m:n:h")) != -1) {
		switch (opt) {
			case 'm':
				model = Model::modelFactory(optarg);
				detectModel = false;
				break;
			case 'n':
				ann = optarg;
				break;
			case 'h':
				cout << "Usage: " << argv[0] << " [OPTION] [FILE]" << endl
					<< endl << "  -m <model> Specifies which model to use for sorting: Whirl, X, Zipper, FatX or Cloud "
					<< endl << "  -n <ann>   Specifies which artificial neural network to use for identification. '" << ann << "' is used by default"
					<< endl << "  -h         Prints this help message"
					<< endl << endl
					<< "With no FILE, or if FILE is '-', stdin will be used"
					<< endl;
				exit(EXIT_SUCCESS);
				break;
			default: /* '?' */
				cerr << "Usage:  " << argv[0] << " [-n <ann>] [-h] [FILE]" << endl;
				exit(EXIT_FAILURE);
		}
	}

	//Open file, or stdin
	istream* in;
	ifstream file;
	if (optind == argc || *argv[optind] == '-'){
		in = &cin;
	}else{
		file.open(argv[optind]);
		if (file.fail()){
			cerr << "Could not open file: '" << argv[optind] << "'" << endl;
			exit(EXIT_FAILURE);
		}
		in = &file;
	}

	//Parse the gene order permutation
	vector<Gene> g;
	copy(istream_iterator<int>(*in), istream_iterator<int>(),
	     back_inserter(g));
	GeneOrder go(g.begin(),g.end());

	//Identify the model
	ModelIdentifier mi(ann);
	priority_queue<pair<double,Model> > pq = mi.identify(go);
	if (detectModel){
		model = pq.top().second;
	}
	while (pq.size() > 0){
		cout << "Model: " << pq.top().second.name() << " score: " << pq.top().first << endl;
		pq.pop();
	}
	cout << "Using model: " << model.name() << endl;

	cout << "Distance: " << inversionDistance(go) << endl;
	//copy(go.begin(), go.end(), ostream_iterator<int>(cout, " "));
	//cout << endl;

	//Sort
	GeneSorter so;
	GeneSorter::ActionList al = so.sort(go,model);

	//Print the result
	double score = 0;
	for (GeneSorter::ActionList::iterator sa = al.begin(); sa != al.end(); ++sa){
		cout << "Action: " << sa->toString() << " model score: " << model.score(*sa,go) << endl;
		score += model.score(*sa,go);
	}
	cout << "Avg score: " << score / al.size() << endl;

	return EXIT_SUCCESS;
}
